@@ -9,7 +9,7 @@ from typing import List
 
 from cnvrg.modules.project import Project
 from cnvrg.modules.errors import UserError
-from cnvrg.modules.errors import CnvrgError
+from cnvrg.modules.errors import CnvrgError, APINotFound
 from cnvrg.helpers.async_runner import AsyncRunner
 from cnvrg.helpers.env_helper import in_experiment
 from cnvrg.helpers.url_builder_helper import url_join
@@ -114,14 +114,21 @@ class Experiment(CnvrgJob, ChartsMixin):
 
         self.project.sync(**payload)
 
-    def log_artifacts(self, files: list):
-        payload = {
-            "job_type": "Experiment",
-            "job": self.job_slug,
-            "in_exp": "true",
-            "files": files,
-        }
-        self.project.sync(**payload)
+    def log_artifacts(self, files: list, target=None):
+        try:
+            payload = {
+                "job_type": "Experiment",
+                "job_slug": self.job_slug
+            }
+            self.project.put_files(files, message="Log Artifacts Commit", payload=payload, prefix=target)
+        except APINotFound:  # To support old server version
+            payload = {
+                "job_type": "Experiment",
+                "job": self.job_slug,
+                "in_exp": "true",
+                "files": files,
+            }
+            self.project.sync(**payload)
 
     @suppress_exception
     def restart(self, message=None, sync=True):
@@ -192,12 +199,12 @@ class Experiment(CnvrgJob, ChartsMixin):
             git_diff=False, commit: str=None, **kwargs):
         function_command = None
         project = project or Project()
-        if sync_before is None:
-            sync_before = not project.git or git_diff
+
         if project.git:
             git_commit = git_commit or project.get_git_commit()
             output_dir = output_dir or "output"
             git_branch = git_branch or project.get_git_branch()
+            sync_before = git_diff
 
         ## sync before
         if sync_before:
@@ -287,7 +294,7 @@ class Experiment(CnvrgJob, ChartsMixin):
                 **kwargs
             )
             if len(e.get("experiments")) == 1:
-                exp = Experiment(e.get("experiments")[0], owner=project.owner)
+                exp = Experiment(e.get("experiments")[0], owner=project.owner+"/"+project.slug)
                 exp.notify_running()
                 exp.set_as_default()
                 return exp
@@ -345,7 +352,7 @@ class Experiment(CnvrgJob, ChartsMixin):
         else:
             self._log_metric(key, Ys, Xs, grouping, x_axis, y_axis)
 
-    def _log_metric(self, key, Ys: List, Xs: List=None, grouping: List=None, x_axis=None, y_axis=None) -> None:
+    def _log_metric(self, key, Ys, Xs, grouping: List=None, x_axis=None, y_axis=None) -> None:
         """
         a function which can tag an experiment with a chart
         :param key: the name of the chart
@@ -356,12 +363,31 @@ class Experiment(CnvrgJob, ChartsMixin):
         :param y_axis:rename the y_axis of the chart
         :return:
         """
+
+        # Support for numpy arrays
         if isinstance(Xs, np.ndarray):
             Xs = Xs.tolist()
         if isinstance(Ys, np.ndarray):
             Ys = Ys.tolist()
 
-        if Xs is None or not isinstance(Xs, list):
+        # Support for arrays with numpy values
+        if isinstance(Ys, list):
+            for index, item in enumerate(Ys):
+                if isinstance(item, np.generic):
+                    Ys[index] = item.item()
+
+        if isinstance(Xs, list):
+            for index, item in enumerate(Xs):
+                if isinstance(item, np.generic):
+                    Xs[index] = item.item()
+
+        # Support for regular types
+        if not isinstance(Ys, list):
+            if isinstance(Ys, np.generic):
+                Ys = Ys.item()
+            Ys = [Ys]
+
+        if Xs is None:
             if key in self.chart_metrics:
                 x_stamp = self.chart_metrics[key]
             else:
@@ -370,6 +396,11 @@ class Experiment(CnvrgJob, ChartsMixin):
             Xs = [*range(x_stamp, x_stamp + len(Ys))]
             self.chart_metrics[key] += len(Ys)
 
+        if not isinstance(Xs, list):
+            if isinstance(Xs, np.generic):
+                Xs = Xs.item()
+            Xs = [Xs]
+
         tag_data = {
             "ys": Ys,
             "xs": Xs,
@@ -393,6 +424,38 @@ class Experiment(CnvrgJob, ChartsMixin):
             ## filter unknown logs?
             [callback(l) if l else None for l in job_logs]
 
+    @staticmethod
+    def list(project, owner=None, flow_id="", flow_version_id=""):
+        owner_slug = owner or apis_helper.credentials.owner
+        if isinstance(project, Project):
+            p = project
+        else:
+            p = Project(project)
+
+        resp = apis_helper.get_v2(
+            url_join(
+                owner_slug,
+                "projects",
+                p.slug,
+                "experiments",
+                "list?filter[flow_slug]={}&filter[flow_version_slug]={}".format(flow_id, flow_version_id)
+            )
+        )
+        return resp.json()
+
+    def commits(self, latest=False):
+        resp = apis_helper.get_v2(url_join(self._base_url(api="v2"), "commits?filter[latest]={}".format(latest)))
+        return resp.json()
+
+    def alert(self, message, subject="You have received alert from cnvrg", recipients=None):
+        if recipients is None:
+            recipients = []
+        resp = apis_helper.post_v2(url_join(self._base_url(api="v2"), "alert"), data={
+            "message": message,
+            "subject": subject,
+            "recipients": recipients
+        })
+        return resp.json()
 
     @property
     def title(self):
@@ -441,16 +504,18 @@ class Experiment(CnvrgJob, ChartsMixin):
         else:
             logger_helper.log_message("You can find the experiment in: {href}".format(href=self.href()))
 
-    @suppress_exception
     def __get_experiment(self):
         resp = apis_helper.get(self._base_url())
-        if not resp: return None
+        if not resp:
+            return None
+        if resp.get('status') == 400:
+            raise CnvrgError(resp.get('message'))
         return resp.get("experiment")
 
-    def _base_url(self):
+    def _base_url(self, api="v1"):
         return url_join(
             #### hackish :D
-            self.project.get_base_url(),string_helper.to_snake_case(self.job_type) + "s", self.job_slug
+            self.project.get_base_url(api=api),string_helper.to_snake_case(self.job_type) + "s", self.job_slug
         )
 
     def __getitem__(self, item):
